//  HcFile: High-Contrast Image File Class using 'SGILOGLUV'--
//	Ward/Larson & Leffler's proposed extensions to TIFF 6.0
//	that stores photometrically accurate images in CIE L u' v' format
// 	set to span the entire range of human vision.
//  
//  J. Tumblin 12/5/97
//
//3456789_123456789_123456789_123456789_123456789_123456789_123456789_123456789_

#include "..\..\libtiff\tiffio.h"	// (Leffler's TIFF library; I installed it privately)
#include <stdio.h>
#include <string.h>

//  Color axes
//  From Greg Ward/Larson "ra_tiff" RADIANCE <->LogluvTIFF converter code;
//--------------------------------------------------------------------------
#define  CIE_x_r_NTSC                0.670           // standard NTSC primaries
#define  CIE_y_r_NTSC                0.330
#define  CIE_x_g_NTSC                0.210 
#define  CIE_y_g_NTSC                0.710
#define  CIE_x_b_NTSC                0.140
#define  CIE_y_b_NTSC                0.080
#define  CIE_x_w_NTSC                0.3333          // use true white 
#define  CIE_y_w_NTSC                0.3333

#define  CIE_y_r                0.330
#define  CIE_x_g                0.290
#define  CIE_y_g                0.600
#define  CIE_x_b                0.150
#define  CIE_y_b                0.060
#define  CIE_x_w                0.3333          // use true white
#define  CIE_y_w                0.3333

//  From tif_luv.c (libTIFF sourcecode by Ward/Larson)
//	r =  2.690*xyz[0] + -1.276*xyz[1] + -0.414*xyz[2];
//	g = -1.022*xyz[0] +  1.978*xyz[1] +  0.044*xyz[2];
//	b =  0.061*xyz[0] + -0.224*xyz[1] +  1.163*xyz[2];

// Misc. Function prototypes
//---------------------------
int lastcall(char *str);     	// exit routine for breakpoints, etc.


//-------------------------------// Color system values by HcFile;
#define hcCIE_XYZ		0     
#define hcCIE_LUV		1
#define hcRGB			2
#define hcRGB_NTSC	 	3

#define hcNAMESIZE	256	// max # of characters in filename strings.

class HcFile 
//-------------------------------------------------------------------------
// High Contrast Image File Class; uses TIFF library & logluv codec
// for scanline-based access to a high-contrast image file. 
// Allows an image file to be opened for reading or opened for writing, 
// but not both at the same time.  Private buffer 'tiffbuf' is loaded or
// read by the TIFF library (always uses CIE XYZ encoding), but 
// the public 'scanline' buffer holds float values using the encoding
// given by 'color_sys'.
{
private:
    char fname[hcNAMESIZE];   	// TIFF filename.
    long xsize,ysize;			// TIFF image size in pixels
    long line_num;            	// current scanline number
    double stordNits;			// scale factor(if any) to get cd/m^2 units.
    int color_sys;     			// Color encoding for contents of 'scanline'
    // 							// buffer as given by "#defines" hcRGB etc.
    int r1w2;					// file state; 0=none; 1=read; 2=write.
    uint16 sampleCnt;	   		// # samples/pixel.(RGB=3; grayscale Y=1, etc.)
    float *tiffbuf;				// One scanline of data to/from TIFF file
    //							// (used only by TIFF library--apps should
    //							// use 'scanline' buffer instead!);
    TIFF* h_tiff;				// TIFF library file handle.

public:
    float *scanline;         	// Scanline buffer holding pixels in
    // 			     			// the color system given by 'color_sys'. 
    //							// (apps read/write here, not in tiffbuf)
public:
    HcFile(void);				// Benign constructor
    ~HcFile(void);				// destructor releases scanline memory;
    //							// read private class members:
    int getColorSys(void){return(color_sys);};
    int getSampleCnt(void){return(sampleCnt);};
    void putColorSys(int nu){color_sys=nu;};
    long getXsize(void){return(xsize);};
    long getYsize(void){return(ysize);};
    long getLineNum(void){return(line_num);};
    double getStoNits(void){return(stordNits);};
    //-----------------------  
    int readOpen(char *filename);// Open TIFF logluv file for reading.
    int readLine(long linenum);	//  read file at linenum; return 1 at >=EOF
    //							//   (use close() when finished with file).
    //----------------------	// Open TIFF logluv file for writing
    //					// (CAREFUL! destroys all previous contents if
    //			     	// file already exists!).
    int writeOpen(char *filename,long sizeX, long sizeY,double stoNits);
    int writeLine(long linenum);// write scanline buffer to file at linenum; 
    int close(void);			// close file; zero-fill if needed.
    
 private://(for use only by HcFile member functions).
    void toTiffbuf(void);		// fill tiffbuf with CIE X,Y,Z pixel values
    //							// calc'd from contents of 'scanline'
    void fromTiffbuf(void);		// fill 'scanline' buffer from CIE X,Y,Z pixel
    //							// values stored in 'tiffbuf'
};
